using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using JALInterfaces;
using System.Reflection;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
//using System.Runtime.Serialization.Formatters.Soap;
using System.Runtime.Serialization;
using System.Globalization;


namespace TestDrawFactory
{	
	/// <summary>
	/// Summary description for Form1.
	/// Demonstrates the use of growable class factories
	/// </summary>

	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		// class properties

		// version properties note use of const
		const int MAJOR_VERSION= 0;
		const int MINOR_VERSION= 1;
		const int BUILD= 1;
		const int UPDATE_VERSION= 0;
		const string SERIALIZATION_NAME= "DrawSerialization";
		private Version serializationVersion= new Version(MAJOR_VERSION,
			MINOR_VERSION,
			BUILD,
			UPDATE_VERSION,
			SERIALIZATION_NAME);

		// default DrawableShape properties
		private Color color= Color.Black;
		private int brushWidth= 2;
		private int size= 50;
		private Point position= new Point(0,0);
		private String currentShape= "Circle";

		// default menu constants
		private readonly int BRUSH_WIDTH_THIN= 1;
		private readonly int BRUSH_WIDTH_MEDIUM= 2;
		private readonly int BRUSH_WIDTH_THICK= 3;
		private readonly int SIZE_SMALL= 20;
		private readonly int SIZE_MEDIUM = 40;
		private readonly int SIZE_LARGE= 60;

		// create our empty factories
		private SerializableDrawableFactory typeFactory= 
			new SerializableDrawableFactory();

		// store array of drawings to refresh screen on Invalidate
		// ASSERT only contains non null IDrawableShapes
		private ArrayList drawHistory= new ArrayList();
		private System.Windows.Forms.MenuItem menuItemSave;

		// peristence support
		private string pathToFile= null;
		private bool isDirtyDocument= false;

		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItemFile;
		private System.Windows.Forms.MenuItem menuItemClear;
		private System.Windows.Forms.MenuItem menuItemExit;
		private System.Windows.Forms.MenuItem menuItemShape;
		private System.Windows.Forms.MenuItem menuItemColor;
		private System.Windows.Forms.MenuItem menuItemColorBlack;
		private System.Windows.Forms.MenuItem menuItemColorRed;
		private System.Windows.Forms.MenuItem menuItemColorGreen;
		private System.Windows.Forms.MenuItem menuItemColorBlue;
		private System.Windows.Forms.MenuItem menuItemSize;
		private System.Windows.Forms.MenuItem menuItemSizeSmall;
		private System.Windows.Forms.MenuItem menuItemSizeMedium;
		private System.Windows.Forms.MenuItem menuItemSizeLarge;
		private System.Windows.Forms.MenuItem menuItemBrush;
		private System.Windows.Forms.MenuItem menuItemBrushThin;
		private System.Windows.Forms.MenuItem menuItemBrushMedium;
		private System.Windows.Forms.MenuItem menuItemBrushThick;
		private System.Windows.Forms.MenuItem menuItemHelp;
		private System.Windows.Forms.MenuItem menuItemHelpAbout;		
		private System.Windows.Forms.MenuItem menuItemPrint;
		private System.Windows.Forms.PrintDialog printDialog1;
		private System.Drawing.Printing.PrintDocument printDocument1;
		private System.Windows.Forms.MenuItem menuItemPrintPreview;
		private System.Windows.Forms.PrintPreviewDialog printPreviewDialog1;		
		private System.Windows.Forms.PageSetupDialog pageSetupDialog1;
		private System.Windows.Forms.MenuItem menuItemPageSetup;
		private System.Windows.Forms.MenuItem menuItemSaveAs;
		private System.Windows.Forms.MenuItem menuItemOpen;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			// add Custom event handlers
			this.MouseUp +=new MouseEventHandler(Form1_MouseUp);
			this.Closing += new CancelEventHandler(this.Form1_Closing);

			// add Statically Compiled classes to our factories
			// type based factory
			typeFactory.Add(typeof(Circle));
			typeFactory.Add(typeof(Square));

			// now dynamically load classes in a "plugin" folder which
			// implement JALInterfaces.IDrawableShape 
			Assembly assembly= null;
			try 
			{      
				// tell app where to look for plugins
				AppDomain.CurrentDomain.AppendPrivatePath("plugins");

				// get absolute path to our private assemblies
				string path= AppDomain.CurrentDomain.BaseDirectory+"plugins";
				
				// create plugins folder if one does not exist
				DirectoryInfo info= new DirectoryInfo(path);
				if (!info.Exists){ info.Create();}
				// discover all dlls in plugins folder
				string[] dir= Directory.GetFiles(path,"*.dll");
				// iterate over files in folder plugins
				foreach (string s in dir)
				{
					string dll= Path.GetFileNameWithoutExtension(s);
					assembly= Assembly.Load(dll);  // in folder plugins
					Type[] types= assembly.GetTypes();
					foreach(Type t in types) 
					{
						//Console.WriteLine("Type: {0}",t);
						try 			
						{
							// only load into type based factory
							// if type implements IDrawableShape
							if (typeof(IDrawableShape).IsAssignableFrom(t)
									&& (t != null) 
									&& ((t.Attributes & TypeAttributes.Serializable) != 0)) 
							{
								typeFactory.Add(t);
							}
						}
						catch(Exception e) 
						{
							Console.WriteLine(e);
						}
					}
				}
				// Done loading factory

				// Populate Shape Menu by
				// iterating over objects in our type based factory
				
				ICollection collection= typeFactory.Keys();
				IEnumerator enumerator= collection.GetEnumerator();
				while (enumerator.MoveNext())
				{
					String menuName= (string)enumerator.Current;
					menuItemShape.MenuItems.Add(menuName,
						new EventHandler(this.shape_Click));
				}

			}
			catch(Exception e) 
			{
				Console.WriteLine(e);
			}
		}

		// Use array of DrawableShape objects to repaint
		// and restore screen state on invalidate. Each object
		// stores its own state and faithfully redraws itself! I believe
		// the framework is smart enough to limit repainting
		// to "dirty" area of screen rather than repaint
		// the entire screen.
		protected override void OnPaint(PaintEventArgs e) 
		{
			Graphics g= e.Graphics;
			foreach (IDrawableShape ds in drawHistory) 
			{
				ds.DrawYourself(g);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Form1));
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItemFile = new System.Windows.Forms.MenuItem();
			this.menuItemClear = new System.Windows.Forms.MenuItem();
			this.menuItemOpen = new System.Windows.Forms.MenuItem();
			this.menuItemSave = new System.Windows.Forms.MenuItem();
			this.menuItemSaveAs = new System.Windows.Forms.MenuItem();
			this.menuItemPageSetup = new System.Windows.Forms.MenuItem();
			this.menuItemPrint = new System.Windows.Forms.MenuItem();
			this.menuItemPrintPreview = new System.Windows.Forms.MenuItem();
			this.menuItemExit = new System.Windows.Forms.MenuItem();
			this.menuItemShape = new System.Windows.Forms.MenuItem();
			this.menuItemColor = new System.Windows.Forms.MenuItem();
			this.menuItemColorBlack = new System.Windows.Forms.MenuItem();
			this.menuItemColorRed = new System.Windows.Forms.MenuItem();
			this.menuItemColorGreen = new System.Windows.Forms.MenuItem();
			this.menuItemColorBlue = new System.Windows.Forms.MenuItem();
			this.menuItemSize = new System.Windows.Forms.MenuItem();
			this.menuItemSizeSmall = new System.Windows.Forms.MenuItem();
			this.menuItemSizeMedium = new System.Windows.Forms.MenuItem();
			this.menuItemSizeLarge = new System.Windows.Forms.MenuItem();
			this.menuItemBrush = new System.Windows.Forms.MenuItem();
			this.menuItemBrushThin = new System.Windows.Forms.MenuItem();
			this.menuItemBrushMedium = new System.Windows.Forms.MenuItem();
			this.menuItemBrushThick = new System.Windows.Forms.MenuItem();
			this.menuItemHelp = new System.Windows.Forms.MenuItem();
			this.menuItemHelpAbout = new System.Windows.Forms.MenuItem();
			this.printDialog1 = new System.Windows.Forms.PrintDialog();
			this.printDocument1 = new System.Drawing.Printing.PrintDocument();
			this.printPreviewDialog1 = new System.Windows.Forms.PrintPreviewDialog();
			this.pageSetupDialog1 = new System.Windows.Forms.PageSetupDialog();
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItemFile,
																					  this.menuItemShape,
																					  this.menuItemColor,
																					  this.menuItemSize,
																					  this.menuItemBrush,
																					  this.menuItemHelp});
			// 
			// menuItemFile
			// 
			this.menuItemFile.Index = 0;
			this.menuItemFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemClear,
																						 this.menuItemOpen,
																						 this.menuItemSave,
																						 this.menuItemSaveAs,
																						 this.menuItemPageSetup,
																						 this.menuItemPrint,
																						 this.menuItemPrintPreview,
																						 this.menuItemExit});
			this.menuItemFile.Text = "File";
			// 
			// menuItemClear
			// 
			this.menuItemClear.Index = 0;
			this.menuItemClear.Text = "Clear Screen";
			this.menuItemClear.Click += new System.EventHandler(this.menuItemClear_Click);
			// 
			// menuItemOpen
			// 
			this.menuItemOpen.Index = 1;
			this.menuItemOpen.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.menuItemOpen.Text = "&Open...";
			this.menuItemOpen.Click += new System.EventHandler(this.menuItemOpen_Click);
			// 
			// menuItemSave
			// 
			this.menuItemSave.Index = 2;
			this.menuItemSave.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.menuItemSave.Text = "&Save";
			this.menuItemSave.Click += new System.EventHandler(this.menuItemSave_Click);
			// 
			// menuItemSaveAs
			// 
			this.menuItemSaveAs.Index = 3;
			this.menuItemSaveAs.Text = "Save As...";
			this.menuItemSaveAs.Click += new System.EventHandler(this.menuItemSaveAs_Click);
			// 
			// menuItemPageSetup
			// 
			this.menuItemPageSetup.Index = 4;
			this.menuItemPageSetup.Text = "Page Setup...";
			this.menuItemPageSetup.Click += new System.EventHandler(this.menuItemPageSetup_Click);
			// 
			// menuItemPrint
			// 
			this.menuItemPrint.Index = 5;
			this.menuItemPrint.Shortcut = System.Windows.Forms.Shortcut.CtrlP;
			this.menuItemPrint.Text = "&Print Screen";
			this.menuItemPrint.Click += new System.EventHandler(this.menuItemPrint_Click);
			// 
			// menuItemPrintPreview
			// 
			this.menuItemPrintPreview.Index = 6;
			this.menuItemPrintPreview.Text = "Print Preview...";
			this.menuItemPrintPreview.Click += new System.EventHandler(this.menuItemPrintPreview_Click);
			// 
			// menuItemExit
			// 
			this.menuItemExit.Index = 7;
			this.menuItemExit.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
			this.menuItemExit.Text = "E&xit";
			this.menuItemExit.Click += new System.EventHandler(this.menuItemExit_Click);
			// 
			// menuItemShape
			// 
			this.menuItemShape.Index = 1;
			this.menuItemShape.Text = "Shape";
			// 
			// menuItemColor
			// 
			this.menuItemColor.Index = 2;
			this.menuItemColor.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.menuItemColorBlack,
																						  this.menuItemColorRed,
																						  this.menuItemColorGreen,
																						  this.menuItemColorBlue});
			this.menuItemColor.Text = "Color";
			// 
			// menuItemColorBlack
			// 
			this.menuItemColorBlack.Index = 0;
			this.menuItemColorBlack.Text = "Black";
			this.menuItemColorBlack.Click += new System.EventHandler(this.menuItemColorBlack_Click);
			// 
			// menuItemColorRed
			// 
			this.menuItemColorRed.Index = 1;
			this.menuItemColorRed.Text = "Red";
			this.menuItemColorRed.Click += new System.EventHandler(this.menuItemColorRed_Click);
			// 
			// menuItemColorGreen
			// 
			this.menuItemColorGreen.Index = 2;
			this.menuItemColorGreen.Text = "Green";
			this.menuItemColorGreen.Click += new System.EventHandler(this.menuItemColorGreen_Click);
			// 
			// menuItemColorBlue
			// 
			this.menuItemColorBlue.Index = 3;
			this.menuItemColorBlue.Text = "Blue";
			this.menuItemColorBlue.Click += new System.EventHandler(this.menuItemColorBlue_Click);
			// 
			// menuItemSize
			// 
			this.menuItemSize.Index = 3;
			this.menuItemSize.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemSizeSmall,
																						 this.menuItemSizeMedium,
																						 this.menuItemSizeLarge});
			this.menuItemSize.Text = "Size";
			// 
			// menuItemSizeSmall
			// 
			this.menuItemSizeSmall.Index = 0;
			this.menuItemSizeSmall.Text = "Small";
			this.menuItemSizeSmall.Click += new System.EventHandler(this.menuItemSizeSmall_Click);
			// 
			// menuItemSizeMedium
			// 
			this.menuItemSizeMedium.Index = 1;
			this.menuItemSizeMedium.Text = "Medium";
			this.menuItemSizeMedium.Click += new System.EventHandler(this.menuItemSizeMedium_Click);
			// 
			// menuItemSizeLarge
			// 
			this.menuItemSizeLarge.Index = 2;
			this.menuItemSizeLarge.Text = "Large";
			this.menuItemSizeLarge.Click += new System.EventHandler(this.menuItemSizeLarge_Click);
			// 
			// menuItemBrush
			// 
			this.menuItemBrush.Index = 4;
			this.menuItemBrush.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.menuItemBrushThin,
																						  this.menuItemBrushMedium,
																						  this.menuItemBrushThick});
			this.menuItemBrush.Text = "Brush";
			// 
			// menuItemBrushThin
			// 
			this.menuItemBrushThin.Index = 0;
			this.menuItemBrushThin.Text = "Thin";
			this.menuItemBrushThin.Click += new System.EventHandler(this.menuItemBrushThin_Click);
			// 
			// menuItemBrushMedium
			// 
			this.menuItemBrushMedium.Index = 1;
			this.menuItemBrushMedium.Text = "Medium";
			this.menuItemBrushMedium.Click += new System.EventHandler(this.menuItemBrushMedium_Click);
			// 
			// menuItemBrushThick
			// 
			this.menuItemBrushThick.Index = 2;
			this.menuItemBrushThick.Text = "Thick";
			this.menuItemBrushThick.Click += new System.EventHandler(this.menuItemBrushThick_Click);
			// 
			// menuItemHelp
			// 
			this.menuItemHelp.Index = 5;
			this.menuItemHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemHelpAbout});
			this.menuItemHelp.Text = "Help";
			// 
			// menuItemHelpAbout
			// 
			this.menuItemHelpAbout.Index = 0;
			this.menuItemHelpAbout.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.menuItemHelpAbout.Text = "&About";
			this.menuItemHelpAbout.Click += new System.EventHandler(this.menuItemAbout_Click);
			// 
			// printDocument1
			// 
			this.printDocument1.PrintPage += new System.Drawing.Printing.PrintPageEventHandler(this.printDocument1_PrintPage);
			// 
			// printPreviewDialog1
			// 
			this.printPreviewDialog1.AutoScrollMargin = new System.Drawing.Size(0, 0);
			this.printPreviewDialog1.AutoScrollMinSize = new System.Drawing.Size(0, 0);
			this.printPreviewDialog1.ClientSize = new System.Drawing.Size(400, 300);
			this.printPreviewDialog1.Enabled = true;
			this.printPreviewDialog1.Icon = ((System.Drawing.Icon)(resources.GetObject("printPreviewDialog1.Icon")));
			this.printPreviewDialog1.Location = new System.Drawing.Point(88, 116);
			this.printPreviewDialog1.MinimumSize = new System.Drawing.Size(375, 250);
			this.printPreviewDialog1.Name = "printPreviewDialog1";
			this.printPreviewDialog1.TransparencyKey = System.Drawing.Color.Empty;
			this.printPreviewDialog1.Visible = false;
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 146);
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = "Draw";
			this.Load += new System.EventHandler(this.Form1_Load);

		}
		#endregion
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		
		private void Form1_Load(object sender, System.EventArgs e)
		{
		
		}

		private void Form1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (isDirtyDocument)
			{
				if (MessageBox.Show ("Really? Quit Without Saving?", 
					"Draw",
					MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question)== DialogResult.No)
				{
					// Cancel the Closing event from closing the form.
					e.Cancel = true;
				}
			}
		}
 
		// trap mouse up and draw shape at mouse position
		// add DrawableShape to ArrayList drawHistory
		// use this ArrayList to refresh screen on Invalidate
		private void Form1_MouseUp(object sender, MouseEventArgs e)
		{
			Graphics g= Graphics.FromHwnd(this.Handle);
			
			if (e.X < 0 || e.Y < 0) 
			{
				g.DrawString("Invalid Mouse Coordinates.",
					new Font("Arial",10),
					new SolidBrush(Color.Red),
					10,
					10,
					StringFormat.GenericDefault);
				return;
			}

			IDrawableShape ds= typeFactory.GetInstance(currentShape);
			if (ds != null) 
			{
				ds.Position= new Point(e.X,e.Y);
				ds.Size= this.size;
				ds.Color= this.color;
				ds.BrushWidth= this.brushWidth;
				ds.DrawYourself(g);
				drawHistory.Add(ds);
				isDirtyDocument= true;
			}
		}

		// will verify in Closing event handler
		private void menuItemExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void menuItemColorBlack_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Black;
			// Menu display may have overwritten drawings 
			// framework calls Invalidate(rect) to repaint screen
		}

		private void menuItemColorRed_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Red;
		}

		private void menuItemColorGreen_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Green;
		}

		private void menuItemColorBlue_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Blue;
		}

		private void menuItemSizeSmall_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_SMALL;
		}

		private void menuItemSizeMedium_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_MEDIUM;
		}

		private void menuItemSizeLarge_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_LARGE;
		}

		private void menuItemBrushThin_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_THIN;
		}

		private void menuItemBrushMedium_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_MEDIUM;
		}

		private void menuItemBrushThick_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_THICK;
		}

		private void menuItemClear_Click(object sender, System.EventArgs e)
		{
			if (this.isDirtyDocument)
			{
				if (MessageBox.Show ("Really? Clear Screen Without Saving?", 
					"Draw",
					MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question)== DialogResult.No)
				{
					return;
				}
			}
			drawHistory.Clear();
			isDirtyDocument= false;
			pathToFile= null;
			this.Invalidate(); 
			// clear the entire screen!
		}

		private void menuItemAbout_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show("A Twisted Object Oriented Program by JAL.","About Draw");
		}

		private void shape_Click(object sender, System.EventArgs e)
		{
			currentShape= ((MenuItem)sender).Text;
		}


		// Add support for printing
		private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			Graphics g= e.Graphics;
			foreach (IDrawableShape ds in drawHistory) 
			{
				ds.DrawYourself(g);
			}
		}

		private void menuItemPrint_Click(object sender, System.EventArgs e)
		{
			printDialog1.AllowSomePages = true;
			printDialog1.ShowHelp = true;
			printDialog1.Document = printDocument1;
			DialogResult result = printDialog1.ShowDialog();
			if (result==DialogResult.OK)
			{
				printDocument1.Print();
			}
		}
		private void menuItemPrintPreview_Click(object sender, System.EventArgs e)
		{
			printPreviewDialog1.Document = printDocument1;
			printPreviewDialog1.ShowDialog();
		}

		private void menuItemPageSetup_Click(object sender, System.EventArgs e)
		{
			pageSetupDialog1.Document = printDocument1;
			pageSetupDialog1.Document.DefaultPageSettings.Color = true;
			pageSetupDialog1.ShowDialog();
		}

		// add support for object peristence
		// ? bug in OpenFileDialog which does not wait for user
		// to click OK if user chooses file with "single click" enabled
		private void menuItemOpen_Click(object sender, System.EventArgs e)
		{
			if (isDirtyDocument)
			{
				if (MessageBox.Show ("Really? Open New Document Without Saving?", 
					"Draw",MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question)== DialogResult.No)
				{
					return;
				}
			}
			Stream inStream;
			OpenFileDialog openFileDialog1 = new OpenFileDialog();
			openFileDialog1.InitialDirectory = "" ;
			openFileDialog1.Filter = "draw files (*.jal)|*.jal" ;
			openFileDialog1.FilterIndex = 1 ;
			openFileDialog1.RestoreDirectory = true ;
			if(openFileDialog1.ShowDialog() == DialogResult.OK)
			{
				string pathToFile= openFileDialog1.FileName;
				if (!Deserialize(pathToFile)) 
				{
					MessageBox.Show("File Error.","Error");
					this.pathToFile= null;
				}
				else 
				{
					this.pathToFile= pathToFile;
					this.isDirtyDocument= false;
					this.Invalidate();
				}
			}
		}


		// ? bug in SaveFileDialog which does not wait for user
		// to click OK if user chooses file with "single click" enabled
		// ? bug in SaveFileDialog which does not wait for user
		// to click OK if user chooses file with "single click" enabled
		private void menuItemSaveAs_Click(object sender, System.EventArgs e)
		{
			Stream outStream ;
			SaveFileDialog saveFileDialog1 = new SaveFileDialog();
			saveFileDialog1.Filter = "Draw files (*.jal)|*.jal";
			saveFileDialog1.FilterIndex = 1 ;
			saveFileDialog1.RestoreDirectory = true ;
			if(saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				string pathToFile= saveFileDialog1.FileName;
				if (!Serialize(pathToFile)) 
				{
					MessageBox.Show("File Error.","Error");
					this.pathToFile= null;
				}
				else 
				{
					this.pathToFile= pathToFile;
					this.isDirtyDocument= false;
				}
			}    
		}

		private void menuItemSave_Click(object sender, System.EventArgs e)
		{
			if (!this.isDirtyDocument) 
			{
				return;
			}
			if (this.pathToFile == null)
			{
				menuItemSaveAs_Click(sender, e);
				return;
			} 
			if (!Serialize(pathToFile)) 
			{
				MessageBox.Show("File Error.","Error");
				this.pathToFile= null;
			}
			else
			{ 
				this.isDirtyDocument= false;
			}
	}


		private bool Serialize(string pathToFile)
		{
			Stream outStream= null;
			// returns false if pathToFile is null
			if (!File.Exists(pathToFile)) {return false;}
			outStream = File.OpenWrite(pathToFile);
			if(outStream == null){return false;}
			// Code to write the stream goes here.
			try
			{
				//SoapFormatter formatter= new SoapFormatter();
				// Serialize our application's screen history
				BinaryFormatter formatter = new BinaryFormatter();
				formatter.Serialize(outStream, serializationVersion);
				formatter.Serialize(outStream, drawHistory);
				return true;
			}
			catch (Exception e)
			{
				System.Console.WriteLine(e);
				return false;
			}
			finally
			{
				outStream.Close();
			}
		}


		private bool Deserialize(string pathToFile)
		{
			Stream inStream= null;
			// returns false if pathToFile is null
			if (!File.Exists(pathToFile)) {return false;}
			inStream = File.OpenRead(pathToFile);
			if(inStream == null){return false;}
			try
			{
				BinaryFormatter formatter = new BinaryFormatter();
				//SoapFormatter formatter= new SoapFormatter();
				object o= formatter.Deserialize(inStream);
				if (!(o is Version))
				{
					MessageBox.Show("Invalid File Type.","Error");
					return false;
				}
				Version fileVersion= (Version)o;
				if (this.serializationVersion == fileVersion)
				{
					drawHistory= (ArrayList)formatter.Deserialize(inStream);
				} 
					// else if (otherVersion){handleHere;}
				else
				{
					MessageBox.Show("Cannot Open File Version: "
						+ fileVersion.VersionAsString,"Error");
					return false;
				}
				return true;
			}
			catch (Exception e)
			{
				System.Console.WriteLine(e);
				return false;
			}
			finally
			{
				inStream.Close();
			}
		} 
	}
	
	// create DrawableShapes statically
	[Serializable]
	class Square : AbstractDrawableShape
	{
		public override void DrawYourself(Graphics g)
		{
			Pen p= new Pen(Color,BrushWidth);
			g.DrawRectangle(p,Position.X, Position.Y,Size,Size);
		}
	}
	[Serializable]
	class Circle : AbstractDrawableShape
	{
		public override void DrawYourself(Graphics g)
		{
			Pen p= new Pen(Color,BrushWidth);
			g.DrawEllipse(p,Position.X,Position.Y,Size,Size);
		}
	}

	// the class factory
	class SerializableDrawableFactory 
	{
		private Hashtable hash= new Hashtable();
		public bool Add(Type t) 
		{
			if ((t != null)
				&& !hash.Contains(t.Name) 
				&& (typeof(IDrawableShape).IsAssignableFrom(t)
				&& ((t.Attributes & TypeAttributes.Serializable) != 0)))
			{
				hash.Add(t.Name,t);
				return true;
			}
			else return false;
		}
		public int Count() 
		{
			return hash.Count;
		}
		public ICollection Keys() 
		{
			return hash.Keys;
		}
		public IDrawableShape GetInstance(string key) 
		{
			if (key == null || !hash.Contains(key)) 
			{
				return null;
			}
			else 
			{
				Type t= (Type)hash[key];
				// dynamically load this class
				return (IDrawableShape)Activator.CreateInstance(t);
			}
		}
		public bool Contains(string key) 
		{
			if (key != null) 
			{
				return hash.Contains(key);
			}
			else {return false;}
		}
	}

	// add support for versionable persistence of complex objects
	interface IVersionable 
	{
		int MajorVersion {get;}
		int MinorVersion {get;}
		int Build {get;}
		int UpdateVersion {get;}
		// Default is [ApplicationName] 
		//[MajorVersion].[MinorVersion].[Build].[UpdateVersion] 
		//[CultureInfo.Name]
		string VersionAsString {get;}
		System.Globalization.CultureInfo CultureInfo {get;}
		string ApplicationName {get;}
	}
	// user should override VersionAsString
	// usage MajorVersion.MinorVersion.Build.Update
	[Serializable]
	public abstract class AbstractVersion : IVersionable 
	{
		private int majorVersion;
		private int minorVersion;
		private int build;
		private int updateVersion;
		private string applicationName;
		private string cultureInfoName;
		private CultureInfo cultureInfo;
		public AbstractVersion(int majorVersion,
			int minorVersion,
			int build,
			int updateVersion,
			string applicationName,
			CultureInfo cultureInfo)
		{
			if (applicationName == null) 
			{
				this.applicationName= "";
			}
			else 
			{
				this.applicationName= applicationName;
			}
			if (cultureInfo == null) 
			{
				this.cultureInfoName= "";
			}
			else 
			{
				this.cultureInfoName= cultureInfo.Name;
			}
			this.majorVersion= majorVersion;
			this.minorVersion= minorVersion;
			this.updateVersion= updateVersion;
			this.build= build;
			this.cultureInfo= cultureInfo;
		}
		// override this, used in Equals, ==, !=
		// MajorVersion.MinorVersion.Build.UpdateVersion
		public virtual string VersionAsString 
		{
			get 
			{
				return this.applicationName
					+ " " + this.majorVersion.ToString()
					+ "." + this.minorVersion.ToString()
					+ "." + this.build
					+ "." + this.updateVersion.ToString()
					+ " " + this.cultureInfoName;			
			}	
		}
		// override GetHashCode()
		public override int GetHashCode() 
		{
			return this.VersionAsString.GetHashCode();
		}
		// override Equals
		public override bool Equals(object o) 
		{
			if (((Version)o).VersionAsString == this.VersionAsString)
			{
				return true;
			}
			else return false;
		}
		// define operator ==
		// called for concrete class!
		// NOT called for references of type object!
		public static bool operator == (AbstractVersion v1, AbstractVersion v2) 
		{
			//System.Console.WriteLine("operator == was called");
			return( v1.Equals(v2));
		}
		// define operator !=
		public static bool operator != (AbstractVersion v1, AbstractVersion v2) 
		{
			return !v1.Equals(v2);
		}
		public int MajorVersion 
		{get {return this.majorVersion;}}
		public int MinorVersion 
		{get {return this.minorVersion;}}
		public int Build {get {return this.build;}
		}
		public int UpdateVersion 
		{get {return this.updateVersion;}}
		public System.Globalization.CultureInfo CultureInfo 
		{get {return this.cultureInfo;}}
		public string ApplicationName 
		{get{return this.applicationName;}}
	}
	// concrete class that uses default VersionToString
	// but passes null for CultureInfo
	[Serializable]
	public class Version : AbstractVersion 
	{
		public Version(int majorVersion,
			int minorVersion,
			int build,
			int updateVersion,
			string applicationName)
			: base(majorVersion,minorVersion,build,updateVersion,
			applicationName,null)
		{;}
	}

}
